var resultLimit = 10
var plateCount = 20000;
var pageOffset = 0;
var pageButtonLimit = 15;

var url = `api/anpr_detectedplates?limit=`;

var licensePlateBeginningTime = document.getElementById("beginningTime");
var licensePlateEndTime = document.getElementById("endingTime");
var searchButton = document.getElementById("searchButton");
var plateSearchInput = document.getElementById("plateSearchInput");
var pageButtonContainer = document.getElementById("pageButtonContainer");
var modalCloseButton = document.getElementById("modalCloseButton")

// Firefox ja safari eivät tue nykyaikaista datetime-local tyyppiä
var isFirefox = navigator.userAgent.indexOf("Firefox") != -1;
var isSafari = (navigator.userAgent.indexOf("Safari") != -1 && navigator.userAgent.indexOf("Chrome")==-1);

if(  isFirefox || isSafari ) {
        licensePlateBeginningTime.type = "date";
        licensePlateEndTime.type = "date";
}

document.addEventListener("DOMContentLoaded",()=> {
        fetchRequest();
        getHeaders();
})

function displayLastButton(data) {
        var lastPageButton = document.getElementById("lastPageButton");
        if ((pageOffset + resultLimit) >= plateCount) {
                lastPageButton.classList = "page-item disabled";
        }
        else if ((pageOffset + resultLimit) <= plateCount) {
                lastPageButton.classList = "page-item";
        }
}

function displayFirstButton(data) {
        var firstPageButtonLi = document.getElementById("firstPageButtonLi")
        if (pageOffset == 0) {
                firstPageButtonLi.classList = "page-item disabled";
        }
        else if (pageOffset != 0) {
                firstPageButtonLi.classList = "page-item";
        } 
}

function displayNextButton(data) {
        var nextButton = document.getElementById("nextButton");
        let currentPage = (Math.floor(pageOffset / resultLimit) + 1);
        let lastPage = Math.floor(data.platecount / resultLimit);
        let lastPageCeil = Math.ceil(data.platecount / resultLimit);
        if(lastPageCeil == currentPage) {
                nextButton.classList = "page-item disabled";
        }
        else {
                nextButton.classList = "page-item";  
        }
}

function displayPreviousButton() {
        previousButton = document.getElementById("previousButton");
        if(pageOffset == 0) {
                previousButton.classList = "page-item disabled";
        }
        else {
                previousButton.classList = "page-item";
        }
}

plateSearchInput.addEventListener("input",() => {
        pageOffset = 0;
        fetchRequest ();
})

function getUrl() {
        let licensePlateEndTimeUnix = getPlateBeginningTimeUnix();
        let licensePlateTimeUnix = getPlateEndTimeUnix();
        let dateToday = (Math.floor(Date.now() / 1000));
        let searchInput = getSearchUrl();
        if(licensePlateEndTimeUnix > dateToday) {
                return ""
        }
        else {
                let compiledUrl = url + resultLimit + "&time=" + licensePlateTimeUnix + "&endtime=" + licensePlateEndTimeUnix + "&offset=" + pageOffset + "&plate=" + searchInput;
                return compiledUrl
        }
}

function getSearchUrl() {
        let searchUrl = "";
        if(plateSearchInput.value != "") {
                searchUrl = plateSearchInput.value;
        }
        return searchUrl
}

function getTimes() {
        //Lisätään varmuudeksi päivä (86400) nykyaikaan, jos kaikkien käyttisten kellot eivät ole oikeassa 
        let endTime = (Math.floor(Date.now() / 1000)) + 86400;
        let beginningTime = 0;
        let checkEndTime = licensePlateEndTime.value;
        let checkBeginningTime = licensePlateBeginningTime.value
        if(checkEndTime != "") {
                endTime = (new Date(licensePlateEndTime.value).getTime() / 1000);
        }
        if(checkBeginningTime != "") {
                beginningTime = (new Date(licensePlateBeginningTime.value).getTime() / 1000);
        }
        return [endTime, beginningTime]
}

function getPlateBeginningTimeUnix() {
        let [endTime, beginningTime] = getTimes();
        licensePlateBeginningTimeUnixs = (Math.min(beginningTime,endTime));
        return licensePlateBeginningTimeUnixs
}

function getPlateEndTimeUnix() {
        let [endTime, beginningTime] = getTimes();
        licensePlateEndTimeUnixs = (Math.max(beginningTime,endTime));
        return licensePlateEndTimeUnixs
}

licensePlateBeginningTime.addEventListener("change",()=> {
        pageOffset = 0;
        fetchRequest ();
})

licensePlateEndTime.addEventListener("change",()=> {
        //Never stop madness ja soon vedetään again ~ A.J
        pageOffset = 0;
        fetchRequest ();
})

function getResultCounter(data, rowHeader) {
        let th = document.createElement("th");
        th.scope = "col";
        if(data.plates.length < resultLimit) {
                th.innerHTML = rowHeader + " " + pageOffset + "-" + data.platecount + "/" + data.platecount;  
        } 
        else {
                th.innerHTML = rowHeader + " " + pageOffset + "-" +  (pageOffset + resultLimit) + " / " + data.platecount;  
        }
        return th;
}

function fetchRequest () {
        paginationPageUrl = getUrl();
        if(paginationPageUrl == "") {      
                let emptyData = {
                        platecount: 0,
                        plates: [],
                        cameras:[],
                }
                updatePlates(emptyData)
        } else {
                fetch(paginationPageUrl,{method: 'GET'})
                        .then(function (response) {
                        return response.json()
                })
                .then(updatePlates);
        }
}

function getCameras(data) {
        let cameraMap = new Map();
        for(let o = 0; o < data.cameras.length; o++) {
                let cameras = data.cameras[o];  
                cameraMap.set(cameras.uuid, cameras.label);
        }
        return cameraMap
}

function getHeaders() {
        let tr = document.createElement("tr");
        let headerTable = document.getElementById("headerTable");
        let rowHeader = ["Time","Date","Plate","Camera"];
        for(let i=0; i < rowHeader.length; i++) {
                let th = document.createElement("th");
                th.innerHTML = rowHeader[i];
                th.scope = "col";
                tr.appendChild(th);
                headerTable.appendChild(tr);
        }  
}

function fetchPlateImage(plates) {
        let imageUrl = "api/snapshots/" + plates.camera
        fetch(imageUrl,{method: 'GET'})
                .then(response => response.blob())
                .then(image => {
                var reader = new FileReader();
                reader.readAsDataURL(image);
                reader.onloadend = function() {
                        var base64data = reader.result;                
                        let image = document.getElementById("image");
                        image.src = base64data;
                }
        }) 
}

function updatePlates(data){
        let table = document.getElementById("licensePlateList");
        table.innerHTML = "";        
        plateCount = data.platecount
        let cameraMap = getCameras(data);
        if(data.plates.length < resultLimit) {
                for(let l = 0;l < data.plates.length; l++) {
                        let tr = document.createElement("tr");
                        let plates = data.plates[l];                        
                        let milliseconds = plates.time * 1000;
                        let dateObject = new Date(milliseconds);
                        let humanTimeFormat = dateObject.toLocaleTimeString();
                        let humanDateFormat = dateObject.toLocaleDateString();  
                        let cameraUuid = plates.camera;     
                        tr.addEventListener("click",() =>{
                                constructModal(humanTimeFormat, humanDateFormat,cameraUuid,plates,cameraMap);
                        })
                        let tl = document.createElement("td");
                        tl.textContent = humanTimeFormat;
                        tr.appendChild(tl);
                        let th = document.createElement("th")
                        th.textContent = humanDateFormat;
                        console.log(humanDateFormat);
                        tr.appendChild(th);
                        let tk = document.createElement("td");
                        tk.textContent = plates.plate;
                        tr.appendChild(tk);
                        let td = document.createElement("td");
                        td.innerHTML = cameraMap.get(cameraUuid);
                        tr.appendChild(td);                
                        table.appendChild(tr)
                }
                for(let l = data.plates.length; l < resultLimit; l++) {
                        let tr = document.createElement("tr");
                        let tl = document.createElement("td");
                        tl.innerHTML = "&nbsp";
                        tr.appendChild(tl);
                        let th = document.createElement("th")
                        th.textContent = "";
                        tr.appendChild(th);
                        let tk = document.createElement("td");
                        tk.textContent = "";
                        tr.appendChild(tk);
                        let td = document.createElement("td");          
                        td.textContent = "";
                        tr.appendChild(td);                
                        table.appendChild(tr)
                }
        }
        else {
                for(let l = 0;l < data.plates.length; l++) {
                        let tr = document.createElement("tr");
                        let plates = data.plates[l];                        
                        let milliseconds = plates.time * 1000;
                        let dateObject = new Date(milliseconds);
                        let humanTimeFormat = dateObject.toLocaleTimeString();
                        // let humanTimeFormat = dateObject.toLocaleTimeString([], {hour: '2-digit', minute:'2-digit'})  
                        let humanDateFormat = dateObject.toLocaleDateString();                
                        let cameraUuid = plates.camera;             
                        tr.addEventListener("click",(args) =>{
                                constructModal(humanTimeFormat, humanDateFormat,cameraUuid,plates,cameraMap);
                        })
                        let tl = document.createElement("td");
                        tl.textContent = humanTimeFormat;
                        tr.appendChild(tl);
                        let th = document.createElement("th")
                        th.textContent = humanDateFormat;
                        tr.appendChild(th);
                        let tk = document.createElement("td");
                        tk.textContent = plates.plate;
                        tr.appendChild(tk);
                        let td = document.createElement("td");
                        td.innerHTML = cameraMap.get(cameraUuid);
                        tr.appendChild(td);                
                        table.appendChild(tr)
                }
        }
        stepsAfterUpdatedPlates(data)
}

function constructModal(humanTimeFormat, humanDateFormat,cameraUuid,plates,cameraMap,args) {
        let modal = new bootstrap.Modal(document.getElementById("popUpModal"));
        let modalImageHeading = document.getElementById("modalImageHeading");
        let image = document.getElementById("image");   
        let modalTitle = document.getElementById("modalTitle");    
        image.innerHTML = fetchPlateImage(plates);
        modalImageHeading.innerHTML = humanTimeFormat + " " + humanDateFormat;
        modalTitle.textContent = cameraMap.get(cameraUuid,cameraUuid);
        modal.show();
}

modalCloseButton.addEventListener("click",()=> {
        let modal = new bootstrap.Modal(document.getElementById("popUpModal"));
        modal.hide()
})

function stepsAfterUpdatedPlates(data) {
        let dateError = false;
        pageNumbers(data); 
        if(data.platecount == 0) {
                previousButton.hidden = true;
                nextButton.hidden = true;
                dateError = true;
                pageNumbers(data,dateError);
        }
        else {
                pageNumbers(data,dateError);   
        }
}

function getPageButton(e,eTextContent,eHighLightedPage,currentPage) {
        let pageButton = document.createElement("li");
        let pageButtonA = document.createElement("a");
        pageButtonA.href = "#"
        if(eHighLightedPage == currentPage){
                pageButton.classList = "page-item active";
                pageButtonA.classList = "page-link";
        }
        else { 
                pageButton.classList = "page-item";
                pageButtonA.classList = "page-link";
        }
        pageButtonA.textContent = eTextContent;  
        pageButton.appendChild(pageButtonA);
        pageButton.id = e; 
        return pageButton
}

function getFirstButton() {
        let firstButton = document.createElement("li");
        let firstButtonA = document.createElement("a");
        firstButtonA.href = "#";
        firstButtonA.textContent = "First";
        firstButtonA.classList = "page-link";
        firstButton.appendChild(firstButtonA);
        firstButton.id = "firstPageButtonLi";
        firstButton.classList  = "page-item";
        firstButton.addEventListener("click",(args) => {
                if(args.target.classList.contains("disabled")) return;
                pageOffset = 0;
                fetchRequest ();
        })
        return firstButton
}

function getPrevioustButton() {
        let previousPageButton = document.createElement("li");
        let previousPageButtonA = document.createElement("a");
        previousPageButtonA.href = "#";
        previousPageButtonA.textContent = "Previous";
        previousPageButtonA.classList = "page-link";
        previousPageButton.appendChild(previousPageButtonA);
        previousPageButton.id = "previousButton";
        previousPageButton.classList  = "page-link";
        previousPageButton.addEventListener("click", (args)=>{;
                if(args.target.classList.contains("disabled")) return;
                else if(pageOffset <= 0) {
                        return
                }
                pageOffset -= resultLimit;
                fetchRequest ();
        })
        return previousPageButton
}

function getNextButton() {
        let nextPageButton = document.createElement("li");
        let nextPageButtonA = document.createElement("a");
        nextPageButtonA.href = "#";
        nextPageButtonA.textContent = "Next";
        nextPageButtonA.classList = "page-link";
        nextPageButton.appendChild(nextPageButtonA);
        nextPageButton.id = "nextButton";
        nextPageButton.classList = "page-item";
        nextPageButton.addEventListener("click", (args)=>{
                if(args.target.classList.contains("disabled")) return;
                else if((pageOffset + (resultLimit - 1)) >= plateCount - 1) {
                        return
                }
                pageOffset += resultLimit;
                fetchRequest ();
        })
        return nextPageButton
}

function getLastButton() {
        let lastButton = document.createElement("li");
        let lastButtonA = document.createElement("a");
        lastButtonA.href = "#";
        lastButtonA.textContent = "Last";
        lastButtonA.classList = "page-link";
        lastButton.appendChild(lastButtonA);
        lastButton.id = "lastPageButton";
        lastButton.classList = "page-item";
        lastButton.addEventListener("click",(args) => {
                if(args.target.classList.contains("disabled")) return;
                else if(plateCount % resultLimit == 0){
                        pageOffset = (plateCount - resultLimit);
                }
                else{
                        pageOffset = resultLimit * (Math.floor(plateCount / resultLimit));
                }
                fetchRequest ();
        })
        return lastButton
}

function getButtons() {
        let firstButton = getFirstButton();
        let previousButton = getPrevioustButton();
        let nextButton = getNextButton();
        let lastButton = getLastButton();
        return [firstButton,previousButton,nextButton,lastButton]

}

function pageNumbers(data,dateError) {
        pageButtonContainer.innerHTML = "";
        if(dateError == true){
                return
        }
        let pageCount = Math.min(pageButtonLimit, Math.ceil(data.platecount / resultLimit));
        let currentPage = (Math.floor(pageOffset / resultLimit) + 1);
        let lastPage = Math.floor(data.platecount / resultLimit);
        let lastPageCeil = Math.ceil(data.platecount / resultLimit);
        let pageNumberSet = ((Math.floor(currentPage / pageButtonLimit)) * pageButtonLimit);
        let lastSetFirstValue = ((Math.floor(lastPage / pageButtonLimit)) * pageButtonLimit);
        let nextPages = ((Math.floor(currentPage / pageButtonLimit)) * pageButtonLimit) + pageButtonLimit;
        let [firstButton, previousPageButton,nextPageButton,lastButton] = getButtons();                
        pageButtonContainer.appendChild(firstButton);
        pageButtonContainer.appendChild(previousPageButton);

//Jos sivuja on vain yksi
        if(lastPage === 0) {

                for(let e = (pageNumberSet + 1); e <= (lastPage + 1) ; e++){
                        let eTextContent = e;
                        let eHighLightedPage = e;
                        let pageButton = getPageButton(e,eTextContent,eHighLightedPage,currentPage);
                        pageButton.addEventListener("click",() => {
                                pageOffset = (pageButton.id * resultLimit)
                                fetchRequest()
                        })
                pageButtonContainer.appendChild(pageButton);
                }
        }
//Jos viimeinen sivu on pageButtonLimit jaollinen
        else if((lastPageCeil % pageButtonLimit) == 0) {
                for(let e = (lastPageCeil - pageButtonLimit); e < (lastPageCeil) ; e++){
                        let eTextContent = e + 1;
                        let eHighLightedPage = e + 1;
                        let pageButton = getPageButton(e,eTextContent,eHighLightedPage,currentPage);
                        pageButton.addEventListener("click",() => {
                                pageOffset = (pageButton.id * resultLimit)
                                fetchRequest()
                        })
                pageButtonContainer.appendChild(pageButton);
                }
        }
//Jos sivuja on vähemmän kuin sivunumerojen maksimi määrä
        else if(lastPage < pageButtonLimit) {
                for(let e = (pageNumberSet); e < (lastPageCeil) ; e++){
                        let eTextContent = e + 1;
                        //Sivutus alkaa ykkösestä
                        let eHighLightedPage = e + 1;
                        let pageButton = getPageButton(e,eTextContent,eHighLightedPage,currentPage);
                        pageButton.addEventListener("click",() => {
                                pageOffset = (pageButton.id * resultLimit)
                                fetchRequest()
                        })
                pageButtonContainer.appendChild(pageButton);
                }
        }
//Jos ollaan viimeisellä sivusetillä
        else if((currentPage + pageButtonLimit) > lastSetFirstValue && lastSetFirstValue == pageNumberSet) {
                for(let e = (pageNumberSet - 1); e <= lastPageCeil; e++){
                        let eTextContent = e;
                        let eHighLightedPage = e;
                        let pageButton = getPageButton(e,eTextContent,eHighLightedPage,currentPage);
                        pageButton.addEventListener("click",() => {
                                pageOffset = ((pageButton.id * resultLimit) - resultLimit)
                                fetchRequest()
                        })
                pageButtonContainer.appendChild(pageButton);
                }
        }
//Kun ollaa jossai sivu välisetissä
        else if(currentPage >= pageButtonLimit) {
                for(let e = (pageNumberSet - 1); e < (nextPages + 1) ; e++){
                        let eTextContent = e;
                        let eHighLightedPage = e;
                        let pageButton = getPageButton(e,eTextContent,eHighLightedPage,currentPage);
                        pageButton.addEventListener("click",() => {
                                pageOffset = ((pageButton.id * resultLimit) - resultLimit)
                                fetchRequest()
                        })
                pageButtonContainer.appendChild(pageButton);
                }
        }
//Jos ollaan ensimmäisellä setillä
        else if (currentPage < pageButtonLimit) {
                for(let e = 0; e < pageCount; e++){
                        //Sivutus alkaa ykkösestä
                        let eTextContent = e + 1;
                        //--||--
                        let eHighLightedPage = e + 1;
                        let pageButton =  getPageButton(e,eTextContent,eHighLightedPage,currentPage);
                        pageButton.addEventListener("click",() => {
                                pageOffset = (pageButton.id * resultLimit)
                                fetchRequest()
                        })
                pageButtonContainer.appendChild(pageButton);
                }
        }
        pageButtonContainer.appendChild(nextPageButton);
        pageButtonContainer.appendChild(lastButton);
        stepsAfterPageNumbers(data)
}

function stepsAfterPageNumbers(data) {
        displayLastButton(data);
        displayFirstButton(data);        
        displayNextButton(data);
        displayPreviousButton();

}
